# Copyright (c) 2005, Brian Frank, Andy Frank, and John Sublett.
# Licensed under BSD, see http://www.fandev.org/license.txt for terms.

#
# makej -- build the Java fan runtime and compiler
#
# @author   Brian Frank
# @creation 23 Dec 05
#

require 'adm/env.rb'

class Makej < Env

  # init
  def initialize
    super
    @temp = File.join(@src_jfan , "temp")
  end

  # main
  def main()
    if (ARGV.include?("clean"))
      clean
    elsif (ARGV.include?("cleanjar"))
      clean
      build
    else
      build
    end
  end
 
  # clean it all up
  def clean
    puts "  makej clean"
    nukeLog(@sys_jar, true)
    nukeLog(@temp, true)
  end
 
  # build the fan.jar 
  def build  
    puts "makej build"
    find
    if (@to_compile.empty?)
      puts "  up-to-date"
      return 
    end
    javac
    jar
  end

  # find all the out of date source packages
  def find()
    base = @src_jfan
    
    # find al the java source files
    java_files = Dir.glob(base + "/**/*.java")

    # strip base directory to get package structure
    java_files.collect! {|f| f[base.size..f.size]}
    
    # remove all the ones that have a newer class file
    java_files = java_files.delete_if do |f| 
      c = "/temp" + f.sub(".java", ".class")
      File.exists?(base+c) && File.mtime(base+c) > File.mtime(base+f)       
    end
    
    # map to packages
    package_dirs = java_files.collect {|f| f[0..f.rindex(File::Separator)] }
    package_dirs.uniq!

    # create list of files to compile (recompile all files in out-of-date packages)
    @packages = package_dirs.collect {|f| f[1..-2].sub("/", ".") }
    @to_compile = package_dirs.collect {|f| base+f+"*.java" }    
  end

  # javac all the source packages
  def javac()
    puts "  compile: " + @packages.join(' ')
    Dir.mkdir(@temp) unless File.exists?(@temp)
    files = @to_compile.join(' ')    
    #cmd = "#@javac -g -cp #@java_home/jre/lib/rt.jar;#@temp -source 1.2 -d #@temp #{files}"
    cmd = "#@javac -g -cp #@java_home/jre/lib/rt.jar;#@temp -d #@temp #{files}"
    r = system(cmd)  
    raise "Cannot compile" unless r
  end

  # jar up the source
  def jar()
    Dir.mkdir(@fan_lib) unless File.exists?(@fan_lib)
    Dir.mkdir(@fan_lib_java) unless File.exists?(@fan_lib_java)
  
    # write manifest
    mf = @temp + "/Manifest.mf"
    File.open(mf, "w") do |f|
      f.puts "Manifest-Version: 1.0"
      f.puts "Main-Class: fanx.tools.Fan"
    end

    puts "  jar: " + @sys_jar
    cmd = "#@jar cfm #@sys_jar #{mf} -C #@temp ."
    r = system(cmd)  
    raise "Cannot jar" unless r
  end

  # cleanup
  def cleanup()
    puts "cleanup"
  end

end

# script
Makej.new.main