# Copyright (c) 2006, Brian Frank, Andy Frank, and John Sublett.
# Licensed under BSD, see http://www.fandev.org/license.txt for terms.

#
# makenative -- build a native module
#
# @author   Brian Frank
# @creation 3 Mar 06
#

require 'adm/env.rb'

class Makenative < Env
  
  # TODO - pod dependencies
  @@depends = { 'webServlet' => ['web'] }

  # main
  def main()
    # check we have at least two args
    if ARGV.size < 2 then
      usage()
      return
    end
    
    # init
    @pod_name = ARGV[1]
    init()
        
    # make
    case ARGV[0]
      when '-j'
        make_java()
      when '-n'
        make_net()
      when '-x'
        make_java()
        make_net()
      else
      usage()
    end    
  end

  # usage
  def usage()
    puts "usage:"
    puts "  makenative -j podName   Make Java natives"
    puts "  makenative -n podName   Make .NET natives"
    puts "  makenative -x podName   Make both Java and .NET natives"
  end
  
  # init
  def init()
    @pod_zip   = File.join(@fan_lib_fan, @pod_name + ".pod")
    @pod_home  = File.join(@fan_src, @pod_name)
    @pod_java  = File.join(@pod_home, "java")
    @pod_net   = File.join(@pod_home, "net")
    @pod_ext   = File.join(@pod_home, "ext")
    @pod_jtemp = File.join(@pod_home, "temp-java")
    @pod_ntemp = File.join(@pod_home, "temp-net")
    bomb("Pod does not exists: " + @pod_name) unless File.exists?(@pod_java)
  end

  # java
  def make_java()
    puts "-- makenative java [" + @pod_name + "]"

    # start with a clean directory    
    nuke(@pod_jtemp)
    Dir.mkdir(@pod_jtemp)
    
    # stub all the dependencies (need to actual declare depends, for
    # now just assume this pod itself)
    cmd = "jstub -d #@pod_jtemp #@pod_name"    
    r = system(cmd)  
    raise "Cannot jstub " unless r
    jstub = "#@pod_jtemp/#@pod_name.jar"

    # find all the java source files
    java_files = Dir.glob(@pod_java + "/**/*.java")
    bomb("No java files found in " + @pod_java) unless java_files.size > 0
    puts java_files.join("\n")

    # map to packages
    package_dirs = java_files.collect {|f| f[0..f.rindex(File::Separator)] }
    package_dirs.uniq!
    to_compile = package_dirs.collect {|f| f+"*.java" }    

    # find any ext jars to compile against
    ext_files = Dir.glob(@pod_ext + "/**/*.jar")

    # find any dependency jars
    dep_files = Array.new
    dep = @@depends[@pod_name]    
    if (dep)
      dep_files = dep.collect {|f| f = File.join(@fan_lib_java, f + ".jar") }
    end

    # compile against stubbed jars
    cp = ["#@java_home/jre/lib/rt.jar",
          "#@sys_jar",
          "#@pod_jtemp/#@pod_name.jar"]
    cp = cp + ext_files + dep_files
    #cmd = "#@javac -g -cp #{cp.join(';')} -source 1.2 -d #@pod_jtemp #{to_compile}"
    cmd = "#@javac -g -cp #{cp.join(';')} -d #@pod_jtemp #{to_compile}"
    puts cmd
    r = system(cmd)  
    raise "Cannot compile" unless r
    
    # extract stub jar into the temp directory
    Dir.chdir(@pod_jtemp) 
    r = system("#@jar -xf #{jstub}");
    raise "Cannot extract jstub" unless r
    
    # we can nuke stub file now (and manifest)
    nuke(jstub)
    nuke("#@pod_jtemp/meta-inf")
 
    # jar everything back up to lib\java\{pod}.jar
    r = system("#@jar cf #@fan_lib_java/#@pod_name.jar -C #@pod_jtemp .");
    raise "Cannot create jar" unless r

    # add class files to pod zip
    add_to_pod(@pod_jtemp)
 
    # cleanup temp
    #nuke(@pod_jtemp)
    puts("-- Success!");
    
  end

  # .net
  def make_net()
    puts "-- makenative net [" + @pod_name + "]"
    puts "    TODO!"
  end
  
  # append files to the pod zip (we use java's jar tool)
  def add_to_pod(path)
    puts "  add_to_path: " + path
    
    cmd = "#@jar -fu #@pod_zip -C #{path} ."
    puts cmd    
    r = system(cmd)  
    raise "Cannot add_to_pod" unless r
  end
  
  # print error message and raise exception
  def bomb(msg)
    puts "FAILED: " + msg
    raise
  end
 
end

# script
Makenative.new.main