//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   10 Jan 06  Brian Frank  Creation
//

**
** BuildCs is the base class for build scripts used to manage
** building C# source code into a .NET exe or dll.
**
abstract class BuildCs : BuildScript
{

//////////////////////////////////////////////////////////////////////////
// Pod Meta-Data
//////////////////////////////////////////////////////////////////////////

  **
  ** Required output file created by the compiler.
  **
  File output := null

  **
  ** Required output type. Possible values are 'exe',
  ** 'winexe', 'library' or 'module'.
  **
  Str target := null

  **
  ** Required list of directories to compile.  All C# source
  ** files in each directory will be compiled.
  **
  File[] dirs := null

  **
  ** List of libraries to link to.
  **
  File[] libs := null

//////////////////////////////////////////////////////////////////////////
// Setup
//////////////////////////////////////////////////////////////////////////

  **
  ** Validate subclass constructor setup required meta-data.
  **
  internal override Void validate()
  {
    ok := true
    ok &= validateReqField("output")
    ok &= validateReqField("target")
    ok &= validateReqField("dirs")
    if (!ok) throw FatalBuildErr.make
  }

//////////////////////////////////////////////////////////////////////////
// Targets
//////////////////////////////////////////////////////////////////////////

  **
  ** BuildPod publishes a default set of targets used
  ** for managing building pods.
  **
  override virtual Target[] targets()
  {
    mine :=
    [
      Target.make(this, "compile", "compile C# source into exe or dll") |,| { compile },
      Target.make(this, "clean",   "delete all intermediate and target files") |,| { clean },
      Target.make(this, "full",    "clean+compile") |,| { full },
    ]
    return mine.addAll(super.targets)
  }

//////////////////////////////////////////////////////////////////////////
// Compile
//////////////////////////////////////////////////////////////////////////

  Void compile()
  {
    log.message("compile", scriptDir.name)
    log.indent

    // compile source
    csc := CompileCs.make(this)
    csc.output = output
    csc.target = target
    csc.src  = dirs
    csc.libs = libs
    csc.run

    log.unindent
  }

//////////////////////////////////////////////////////////////////////////
// Clean
//////////////////////////////////////////////////////////////////////////

  Void clean()
  {
    log.message("clean", scriptDir.name)
    log.indent
    Delete.make(this, output).run
    log.unindent
  }

//////////////////////////////////////////////////////////////////////////
// Full
//////////////////////////////////////////////////////////////////////////

  Void full()
  {
    clean
    compile
  }
}