//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   6 Jun 06  Brian Frank  Creation
//

**
** ImportedType is the implementation of CType for a type imported
** from a precompiled pod (as opposed to a TypeDef within the compilation
** units being compiled).
**
class ImportedType : CType
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  **
  ** Construct with loaded Type.
  **
  new make(Namespace ns, Type t)
  {
    this.ns     = ns
    this.pod    = ns.importPod(t.pod)
    this.t      = t
    this.base   = ns.importType(t.base)
    this.mixins = ns.importTypes(t.mixins)
  }

//////////////////////////////////////////////////////////////////////////
// CType
//////////////////////////////////////////////////////////////////////////

  override Str name()      { return t.name }
  override Str qname()     { return t.qname }
  override Str signature() { return t.signature }

  override Bool isAbstract()  { return t.isAbstract }
  override Bool isConst()     { return t.isConst }
  override Bool isFinal()     { return t.isFinal }
  override Bool isEnum()      { return t.isEnum }
  override Bool isInternal()  { return t.isInternal }
  override Bool isMixin()     { return t.isMixin }
  override Bool isPublic()    { return t.isPublic }
  override Bool isSynthetic() { return t.isSynthetic }

  override Bool isGeneric() { return t.isGeneric }
  override Bool isParameterized() { return !t.params.isEmpty }
  override Bool isGenericParameter() { return pod === ns.sysPod && name.size === 1 }

  override ListType toListOf()
  {
    if (listOf == null) listOf = ListType.make(this)
    return listOf
  }

  override Str:CSlot slots()
  {
    if (slotMap == null)
    {
      slotMap = Str:CSlot[:]
      if (!isGenericParameter)
        t.slots.each |Slot s| { slotMap[s.name] = ns.importSlot(s) }
    }
    return slotMap
  }

  override Str toStr() { return signature() }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  readonly Type t
  override readonly Namespace ns
  override readonly CPod pod
  override readonly CType base
  override readonly CType[] mixins
  private Str:CSlot slotMap
  private ListType listOf

}