//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   5 Jun 06  Brian Frank  Creation
//

**
** InitInput verifies the CompilerInput instance, initializes Comiler.pod
** with a PodDef, and tokenizes the source code from either file input
** or string input.
**
class InitInput : CompilerStep
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  **
  ** Constructor takes the associated Compiler
  **
  new make(Compiler compiler)
    : super(compiler)
  {
  }

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  **
  ** Run the step
  **
  override Void run()
  {
    // validate input
    input := compiler.input
    try
    {
      input.validate
    }
    catch (CompilerErr err)
    {
      throw errReport(err)
    }

    // init pod
    podName := input.podName
    compiler.pod = PodDef.make(ns, Location.make(podName), podName)
    compiler.isSys = podName == "sys"

    // process intput into tokens
    switch (input.mode)
    {
      case CompilerInputMode.str:
        Tokenize.make(compiler).runSource(input.srcStrLocation, input.srcStr)
      case CompilerInputMode.file:
        FindSourceFiles.make(compiler).run
        Tokenize.make(compiler).run
      default:
        throw err("Unknown input mode $input.mode", null)
    }
  }

}