//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   5 May 06  Brian Frank  Creation
//

using compiler

**
** DocCompiler manages the pipeline of compiling API and
** stand-alone fandoc documents into HTML.
**
class DocCompiler
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  new make()
  {
    log = CompilerLog.make
    errors = CompilerErr[,]
    outDir = Sys.homeDir + `doc/`
    uriMapper = UriMapper.make(this)
  }

//////////////////////////////////////////////////////////////////////////
// Compile
//////////////////////////////////////////////////////////////////////////

  Void compileToHtml()
  {
    log.message("DocCompiler [$pod]")
    Init.make(this).run
    ApiToHtml.make(this).run
    SourceToHtml.make(this).run
    FandocToHtml.make(this).run
    CopyResources.make(this).run
    if (!errors.isEmpty) throw errors.last
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  CompilerLog log           // ctor
  CompilerErr[] errors      // accumulated errors
  File outDir               // output directory
  Pod pod                   // pod to compile
  UriMapper uriMapper       // normalizes fandoc URIs to HTML
  File podDir               // Init: outDir/podName

}