//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   5 May 07  Brian Frank  Creation
//

using compiler
using fandoc

**
** FandocToHtmlGenerator generates an HTML file for a standalone fandoc file
**
class FandocToHtmlGenerator : HtmlGenerator
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  new make(DocCompiler compiler, Location loc, OutStream out, Doc doc)
    : super(compiler, loc, out)
  {
    this.doc = doc
  }

//////////////////////////////////////////////////////////////////////////
// Generator
//////////////////////////////////////////////////////////////////////////

  override Void generate()
  {
    doc.write(this)
    out.close
  }

  override Void docHead(Doc doc)
  {
    super.docHead(doc)
    out.print("  <link rel='stylesheet' type='text/css' href='../fandoc.css' />\n")
  }

  override Void elemStart(DocElem elem)
  {
    super.elemStart(elem)
    if (elem.htmlName == "body")
    {
      out.print("<div id='content'>\n")
      out.print("<div id='nav'>\n")
      out.print("<ul>\n")
      out.print("  <li><a href='..'>All Pods</a></li>\n")
      out.print("  <li><a href='.'>$compiler.pod.name Pod</a></li>\n")
      out.print("</ul>\n")
      out.print("</div>\n")
    }
  }

  override Void elemEnd(DocElem elem)
  {
    if (elem.htmlName == "body")
    {
      out.print("</div>\n")
      out.print("<div id='sidebarBg'></div>\n")
      out.print("<div id='sidebar'>\n")
      out.print("</div>\n")
    }
    super.elemEnd(elem)
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  Doc doc

}
