//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   5 May 07  Brian Frank  Creation
//

using compiler
using fandoc

**
** FandocToHtml generates an HTML file for each fandoc file in pod
**
class FandocToHtml : DocCompilerSupport
{

  new make(DocCompiler compiler)
    : super(compiler)
  {
  }

  Void run()
  {
    compiler.pod.files.each |File file|
    {
      if (file.ext == "fandoc")
        generate(file)
    }
  }

  Void generate(File inFile)
  {
    log.verbose("  Fandoc [$inFile]")
    try
    {
      log.verbose("  Fandoc [$inFile]")
      doc := FandocParser.make.parse(inFile.name, inFile.in)
      loc := Location.make(compiler.pod + "::" + inFile.name)

      outFile := compiler.podDir + "${inFile.basename}.html".toUri

      FandocToHtmlGenerator.make(compiler, loc, outFile.out, doc).generate
    }
    catch (Err e)
    {
      errReport(CompilerErr.make("Cannot parse fandoc file", Location.makeFile(inFile), e))
    }
  }
}
