//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jan 06  Brian Frank  Creation
//
package fan.sys;

import fanx.fcode.*;

/**
 * Field is a slot which "stores" a value.
 */
public class Field
  extends Slot
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  public Field(Type parent, Str name, int flags, Facets facets, Type fieldType)
  {
    super(parent, name, flags, facets);
    this.fieldType = fieldType;
  }

//////////////////////////////////////////////////////////////////////////
// Signature
//////////////////////////////////////////////////////////////////////////

  public Type type()  { return Sys.FieldType;  }

  public Type fieldType() { return fieldType; }

  public Method getter() { return getter; }

  public Method setter() { return setter; }

  public Str signature() { return Str.make(fieldType.toStr().val + " " + Str.make(name.val)); }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  public Obj get() { return get(null); }
  public Obj get(Obj instance)
  {
    parent.finish();

    if (getter != null)
    {
      return getter.invoke(instance, Method.noArgs);
    }

    try
    {
      return (Obj)reflect.get(instance);
    }
    catch (Exception e)
    {
      if (reflect == null)
        throw Err.make("Field not mapped to java.lang.reflect correctly").val;

      throw Err.make(e).val;
    }
  }

  public void set(Obj instance, Obj value)
  {
    parent.finish();

    if ((flags & FConst.Const) != 0)
      throw ReadonlyErr.make("Cannot set const field " + qname()).val;

    if (setter != null)
    {
      setter.invoke(instance, new Obj[] { value });
      return;
    }

    try
    {
      reflect.set(instance, value);
    }
    catch (Exception e)
    {
      if (reflect == null)
        throw Err.make("Field not mapped to java.lang.reflect correctly").val;

      throw Err.make(e).val;
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  final Type fieldType;
  Method getter;
  Method setter;
  java.lang.reflect.Field reflect;

}