//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Dec 05  Brian Frank  Creation
//
package fan.sys;

/**
 * StrBuf mutable random-access sequence of integer characters.
 */
public class StrBuf
  extends FanObj
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  /**
   * Create with initial capacity of 16.
   */
  public static StrBuf make()
  {
    return new StrBuf(new StringBuilder(16));
  }

  /**
   * Create with specified capacity.
   */
  public static StrBuf make(Int capacity)
  {
    return new StrBuf(new StringBuilder((int)capacity.val));
  }

  public StrBuf(StringBuilder sb)
  {
    this.sb = sb;
  }

// TODO - nuke in next build
public static StrBuf makeCapacity(Int capacity)
{
  return new StrBuf(new StringBuilder((int)capacity.val));
}

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  /*
  public Bool equals(Obj obj)
  {
    return val.equals(((Str)obj).val) ? Bool.True : Bool.False;
  }
  */

  public Bool isEmpty()
  {
    return sb.length() == 0 ? Bool.True : Bool.False;
  }

  public Int size()
  {
    return Int.pos(sb.length());
  }

  public Int get(Int index)
  {
    int i = (int)index.val;
    if (i < 0) i = sb.length()+i;
    return Int.pos[sb.charAt(i)];
  }

  public StrBuf set(Int index, Int ch)
  {
    int i = (int)index.val;
    if (i < 0) i = sb.length()+i;
    sb.setCharAt(i, (char)ch.val);
    return this;
  }

  public StrBuf add(Obj x)
  {
    String s = (x == null) ? "null" : x.toStr().val;
    sb.append(s);
    return this;
  }

  public StrBuf addChar(Int ch)
  {
    sb.append((char)ch.val);
    return this;
  }

  public StrBuf insert(Int index, Obj x)
  {
    String s = (x == null) ? "null" : x.toStr().val;
    sb.insert((int)index.val, s);
    return this;
  }

  public StrBuf grow(Int size)
  {
    sb.ensureCapacity((int)size.val);
    return this;
  }

  public StrBuf clear()
  {
    sb.setLength(0);
    return this;
  }

  public Str toStr()
  {
    return Str.make(sb.toString());
  }

  public Type type()
  {
    return Sys.StrBufType;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  StringBuilder sb;

}