//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Mar 06  Brian Frank  Creation
//
package fan.sys;

import java.io.*;

/**
 * SysInStream
 */
public class SysInStream
  extends FanObj
  implements InStream
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  public static SysInStream make(java.io.InputStream in, Int bufSize)
  {
    if (bufSize == null || bufSize.val == 0)
      return new SysInStream(in);
    else
      return new SysInStream(new java.io.BufferedInputStream(in, (int)bufSize.val));
  }

  public SysInStream(InputStream in)
  {
    this.in = in;
  }

//////////////////////////////////////////////////////////////////////////
// Obj
//////////////////////////////////////////////////////////////////////////

  public Type type() { return Sys.SysInStreamType; }

//////////////////////////////////////////////////////////////////////////
// InStream
//////////////////////////////////////////////////////////////////////////

  public Int read() { int n = r(); return n < 0 ? null : Int.pos(n); }
  public int r()
  {
    try
    {
      return in.read();
    }
    catch (IOException e)
    {
      throw IOErr.make(e).val;
    }
  }

  public Int readBuf(Buf buf, Int n)
  {
    try
    {
      int len = (int)n.val;
      buf.grow(buf.pos+len);
      int read = in.read(buf.buf, buf.pos, len);
      if (read < 0) return null;
      buf.pos  += read;
      buf.size += read;
      return Int.pos(read);
    }
    catch (IOException e)
    {
      throw IOErr.make(e).val;
    }
  }

  public InStream unread(Int n) { return unread((int)n.val); }
  public InStream unread(int n)
  {
    try
    {
      // don't take the hit until we know we need to wrap
      // the raw input stream with a pushback stream
      if (!(in instanceof PushbackInputStream))
        in = new PushbackInputStream(in, 1024);
      ((PushbackInputStream)in).unread(n);
      return this;
    }
    catch (IOException e)
    {
      throw IOErr.make(e).val;
    }
  }

  public Buf readAllBuf() { return InStream$.readAllBuf(this); }

  public Buf readBufFully(Buf buf, Int n) { return InStream$.readBufFully(this, buf, n); }

  public Int readU1() { return InStream$.readU1(this); }

  public Int readS1() { return InStream$.readS1(this); }

  public Int readU2() { return InStream$.readU2(this); }

  public Int readS2() { return InStream$.readS2(this); }

  public Int readU4() { return InStream$.readU4(this); }

  public Int readS4() { return InStream$.readS4(this); }

  public Int readS8() { return InStream$.readS8(this); }

  public Float readF4() { return InStream$.readF4(this); }

  public Float readF8() { return InStream$.readF8(this); }

  public Bool readBool() { return InStream$.readBool(this); }

  public Str readUtf() { return InStream$.readUtf(this); }

  public int rChar()
  {
    return charsetDecoder.decode(this);
  }

  public Int readChar()
  {
    int ch = charsetDecoder.decode(this);
    return ch < 0 ? null : Int.pos(ch);
  }

/*
  public OutStream writeChars(Str s) { return writeChars(s.val, 0, s.val.length()); }
  public OutStream writeChars(Str s, Int off) { return writeChars(s.val, (int)off.val, s.val.length()-(int)off.val); }
  public OutStream writeChars(Str s, Int off, Int len) { return writeChars(s.val, (int)off.val, (int)len.val); }
  public OutStream writeChars(String s, int off, int len)
  {
    try
    {
      writer().write(s, off, len);
      return this;
    }
    catch (java.io.IOException e)
    {
      throw IOErr.make(e);
    }
  }
*/

  public InStream unreadChar(Int c)
  {
    charsetEncoder.encode((char)c.val, this);
    return this;
  }

  public Str readLine() { return InStream$.readLine(this, Int.pos[4096]); }
  public Str readLine(Int max) { return InStream$.readLine(this, max); }

  public List readAllLines() { return InStream$.readAllLines(this); }

  public Str readAllStr() { return InStream$.readAllStr(this, Bool.True); }
  public Str readAllStr(Bool normalizeNewlines)  { return InStream$.readAllStr(this, normalizeNewlines); }

  public Map readProps() { return InStream$.readProps(this); }

  public Charset charset()
  {
    return charset;
  }

  public void charset(Charset charset)
  {
    this.charsetDecoder = charset.newDecoder();
    this.charsetEncoder = charset.newEncoder();
    this.charset = charset;
  }

  public Int pipe(OutStream out) { return InStream$.pipe(this, out, null, Bool.True); }
  public Int pipe(OutStream out, Int n) { return InStream$.pipe(this, out, n, Bool.True); }
  public Int pipe(OutStream out, Int n, Bool close) { return InStream$.pipe(this, out, n, close); }

  public Bool close()
  {
    try
    {
      in.close();
      return Bool.True;
    }
    catch (IOException e)
    {
      return Bool.False;
    }
  }

//////////////////////////////////////////////////////////////////////////
// Java Conversion
//////////////////////////////////////////////////////////////////////////

  /**
   * Get a java.io.InputStream for the specified input stream.
   */
  public static InputStream java(InStream in)
  {
    if (in instanceof SysInStream)
      return ((SysInStream)in).in;
    else
      return new JavaInputStream(in);
  }

  static class JavaInputStream extends InputStream
  {
    JavaInputStream(InStream in) { this.in = in; }
    public int read()
    {
      Int b = in.read();
      if (b == null) return -1;
      return (int)b.val;
    }
    public int read(byte[] b, int off, int len)
    {
      buf.buf = b;
      buf.pos = off;
      buf.size = b.length;
      Int n = in.readBuf(buf, Int.make(len));
      if (n == null) return -1;
      return (int)n.val;
    }
    public void close() { in.close(); }
    InStream in;
    Buf buf = new Buf();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  InputStream in;
  Charset charset = Charset.utf8();
  Charset.Decoder charsetDecoder = charset.newDecoder();
  Charset.Encoder charsetEncoder = charset.newEncoder();

}