//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler;

import java.util.*;
import fan.sys.*;
import fan.sys.List;
import fanx.compiler.ast.*;

/**
 * VisitStep represents one discrete task run during the compiler pipeline.
 * The implementations are found in fanx.compiler.steps.
 */
public abstract class CompilerStep
  extends CompilerSupport
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  public CompilerStep(Compiler compiler)
  {
    super(compiler);
  }

//////////////////////////////////////////////////////////////////////////
// Overrides
//////////////////////////////////////////////////////////////////////////

  public abstract void run();

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  public void addClassDefs(ArrayList list)
  {
    addClassDefs((ClassDef[])list.toArray(new ClassDef[list.size()]));
  }

  public void addClassDefs(ClassDef[] newClasses)
  {
    int oldCount = compiler.classDefs.length;
    int newCount = oldCount + newClasses.length;
    ClassDef[] cls = new ClassDef[newCount];
    System.arraycopy(compiler.classDefs, 0, cls, 0, oldCount);
    System.arraycopy(newClasses, 0, cls, oldCount, newClasses.length);
    compiler.classDefs = cls;
  }

  public static Slot mapSlot(Type parent, SlotDef def)
  {
    if (def instanceof FieldDef)
      return mapField(parent, (FieldDef)def);
    else
      return mapMethod(parent, (MethodDef)def);
  }

  public static Field mapField(Type parent, FieldDef def)
  {
    return def.field = new Field(parent, Str.make(def.name).intern(), def.flags, null, def.type.type);
  }

  public static Method mapMethod(Type parent, MethodDef def)
  {
    List params = new List(Sys.MethodParamType, def.params.size());
    for (int i=0; i<def.params.size(); ++i)
    {
      MethodParamDef pdef = (MethodParamDef)def.params.get(i);
      int pflags = pdef.def == null ? 0 : MethodParam.HAS_DEFAULT;
      params.add(new MethodParam(Str.make(pdef.name).intern(), pdef.type.type, pflags));
    }
    return def.method = new Method(parent, Str.make(def.name).intern(), def.flags, null, def.ret.type, def.ret.type, params);
  }

}