//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Apr 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fan.sys.*;
import fanx.compiler.*;

/**
 * EnumDef is used to define one ordinal/named enum value in an enum ClassDef.
 */
public class EnumDef
  extends Node
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public EnumDef(ILocation location, int ordinal, String name)
  {
    super(location);
    this.ordinal = ordinal;
    this.name = name;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  public Node walk(Visitor v)
  {
    ctorArgs = walk(v, ctorArgs);
    return this;
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "EnumDef " + ordinal + ":" + name;
  }

  public void tree(AstWriter out)
  {
    out.w(this).nl();
  }

  public void pretty(AstWriter out)
  {
    out.w(name).nl();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public int ordinal;
  public String name;
  public ArrayList ctorArgs = new ArrayList();  // Expr[]

}