//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   25 Nov 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * LocalVarDefStmt models a local variable declaration and it's
 * optional initialization expression.
 */
public class LocalVarDefStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public LocalVarDefStmt(ILocation location, TypeRef type, String name, Expr init)
  {
    super(location);
    this.type = type;
    this.name = name;
    this.init = init;
  }

  public LocalVarDefStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return LOCAL_VAR_DEF;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    type = walk(v, type);
    init = walk(v, init);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    String s = type + " " + name;
    if (init != null) s += " := " + init;
    return s;
  }

  public void tree(AstWriter out)
  {
    out.w("LocalVarDefStmt ").w(name).nl();
    out.indent();
    if (type != null) { out.w("type: "); type.tree(out); }
    if (init != null) { out.w("init: "); init.tree(out); }
    out.unindent();
  }

  public void pretty(AstWriter out, boolean endOfStmt)
  {
    if (type != null) { type.pretty(out); out.w(" "); }
    out.w(name);
    if (init != null) { out.w(" := "); init.pretty(out); }
    if (endOfStmt) out.w(";").nl();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public TypeRef type;      // type of the variable
  public String name;       // variable name
  public Expr init;         // initialization assignment expression or null
  public MethodVar var;     // used to bind to a specific register
  public boolean catchVar;  // true if this is a catch (Err err)

}