//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.compiler.*;

/**
 * LocalVarExpr puts a local variable onto the stack.  It references
 * a MethodVar which actually defines the variable and it's register.
 */
public class LocalVarExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public LocalVarExpr(ILocation location, MethodVar var)
  {
    super(location);
    this.type = var.type;
    this.var  = var;
  }

  public LocalVarExpr(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// LocalVarExpr
//////////////////////////////////////////////////////////////////////////

  public int register()
  {
    return var.register;
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return LOCAL_VAR;
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return var.name;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public MethodVar var;

  // used to mark a local var access that's should not be pulled out
  // into cvars, even if var.usedInClosure is true
  public boolean noRemapToCvars;

}