//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   17 Mar 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.compiler.*;

/**
 * MapTypeRef is used to represent a parameterized Map type.
 */
public class MapTypeRef
  extends TypeRef
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public MapTypeRef(ILocation location, TypeRef key, TypeRef val)
  {
    super(location, "sys::Map");
    this.key = key;
    this.val = val;
  }

//////////////////////////////////////////////////////////////////////////
// MapTypeRef
//////////////////////////////////////////////////////////////////////////

  public MapType toMapType()
  {
    if (key.type == null || val.type == null) throw new IllegalStateException();
    return new MapType(key.type, val.type);
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  public Node walk(Visitor v)
  {
    key = walk(v, key);
    val = walk(v, val);
    return v.visit(this);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    if (type != null) return type.toString();
    return val + "[" + key + "]?";
  }

  public void tree(AstWriter out)
  {
    out.w("MapTypeRef " + toString()).nl();
  }

  public void pretty(AstWriter out)
  {
    out.w(toString());
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public TypeRef key;
  public TypeRef val;

}