//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   24 May 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.ArrayList;
import fan.sys.*;
import fanx.compiler.*;

/**
 * SwitchCase models a series of case labels and their
 * associated code block in a SwitchStmt.
 */
public class SwitchCase
  extends Node
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public SwitchCase(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  public Node walk(Visitor v)
  {
    cases = walk(v, cases);
    block = walk(v, block);
    return v.visit(this);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("SwitchCase").nl();
    out.indent();
    out.w("cases: ").nl();
    out.indent();
    for (int i=0; i<cases.size(); ++i)
    {
      ((Expr)cases.get(i)).tree(out);
    }
    out.unindent();
    if (block != null) { out.w("block: "); block.tree(out); }
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    for (int i=0; i<cases.size(); ++i)
    {
      Expr expr = (Expr)cases.get(i);
      out.w("case " + expr + ":").nl();
    }
    out.indent();
    if (block != null) block.pretty(out, false);
    out.unindent();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public ArrayList cases = new ArrayList();   // Expr[]
  public Block block;
  public int startOffset;    // start offset in CodeAsm

}