//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   23 Jun 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fanx.compiler.*;
import fanx.compiler.parser.*;

/**
 * TernaryExpr is used for the ternary expression <cond> ? <true> : <false>
 */
public class TernaryExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public TernaryExpr(ILocation location, Expr condition, Expr trueExpr, Expr falseExpr)
  {
    super(location);
    this.condition = condition;
    this.trueExpr  = trueExpr;
    this.falseExpr = falseExpr;
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return TERNARY;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    condition = walk(v, condition);
    trueExpr  = walk(v, trueExpr);
    falseExpr = walk(v, falseExpr);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "(" + condition + ") ? (" + trueExpr + ") : (" + falseExpr + ")";
  }

  public void tree(AstWriter out)
  {
    out.w("TernaryExpr " ).w(" " + leaveToString()).nl();
    out.indent();
    out.w("condition: "); condition.tree(out);
    out.w("trueExpr:  "); trueExpr.tree(out);
    out.w("falseExpr: "); falseExpr.tree(out);
    out.unindent();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr condition;     // boolean test
  public Expr trueExpr;      // result of expression if condition is true
  public Expr falseExpr;     // result of expression if condition is false

}