//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * Throw an exception.
 */
public class ThrowStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public ThrowStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return THROW;
  }

  public boolean isMethodExit()
  {
    return true;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    exception = walk(v, exception);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "throw " + exception + ";";
  }

  public void tree(AstWriter out)
  {
    out.w("ThrowStmt").nl();
    out.indent();
    out.w("exception: "); exception.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("throw ");
    exception.pretty(out);
    out.w(";").nl();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr exception;      // expression which leaves exception on stack

}