//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.compiler.*;

/**
 * TypeVarExpr models a type name variable such as Math or java.lang.Math.
 * It is used to scope a static method.
 */
public class TypeVarExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public TypeVarExpr(ILocation location, Type type)
  {
    super(location);
    this.type = type;
    this.typeRef = new TypeRef(location, type);
  }

  public TypeVarExpr(ILocation location, TypeRef typeRef)
  {
    super(location);
    this.typeRef = typeRef;
  }

  public TypeVarExpr(ILocation location, String name)
  {
    super(location);
    this.typeRef = new TypeRef(location, name);
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return TYPE_VAR;
  }

  public TypeRef toTypeRef()
  {
    return typeRef;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    typeRef = walk(v, typeRef);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    if (type != null) return type.toString();
    else return typeRef.toString();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public TypeRef typeRef;
}