//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

/**
 * Vistor is used to visit each node in the abstract syntax tree.
 */
public abstract class Visitor
{

//////////////////////////////////////////////////////////////////////////
// Visit
//////////////////////////////////////////////////////////////////////////

  /**
   * Visit the specified node.  Return the node you wish to replace
   * the specified node with in the abstract tree (typically the
   * same node).
   */
  public abstract Node visit(Node node);

//////////////////////////////////////////////////////////////////////////
// Scoping
//////////////////////////////////////////////////////////////////////////

  /**
   * Callback when entering a class.
   */
  public void enterClassDef(ClassDef def) {}

  /**
   * Callback when exiting a class.
   */
  public void exitClassDef(ClassDef def) {}

  /**
   * Callback when entering a field.
   */
  public void enterFieldDef(FieldDef def) {}

  /**
   * Callback when exiting a field.
   */
  public void exitFieldDef(FieldDef def) {}

  /**
   * Callback when entering a method.
   */
  public void enterMethodDef(MethodDef def) {}

  /**
   * Callback when exiting a method.
   */
  public void exitMethodDef(MethodDef def) {}

  /**
   * Callback when entering a stmt.
   */
  public void enterStmt(Stmt stmt) {}

  /**
   * Callback when exiting a stmt.
   */
  public void exitStmt(Stmt stmt) {}

  /**
   * Callback when entering a block.
   */
  public void enterBlock(Block block) {}

  /**
   * Callback when exiting a block.
   */
  public void exitBlock(Block block) {}

  /**
   * Callback when entering a expr.
   */
  public void enterExpr(Expr expr) {}

  /**
   * Callback when exiting a expr.
   */
  public void exitExpr(Expr expr) {}

}