//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   9 Mar 06  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.util.*;
import fan.sys.*;
import fanx.fcode.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;
import fanx.compiler.parser.*;

/**
 * DefaultCtor adds a default public constructor called make() if
 * no constructor was explicitly specified.
 */
public class DefaultCtor
  extends CompilerStep
{

  public DefaultCtor(Compiler compiler)
  {
    super(compiler);
  }

  public void run()
  {
    for (int i=0; i<compiler.classDefs.length; ++i)
      checkClass(compiler.classDefs[i]);
  }

  void checkClass(ClassDef cls)
  {
    if (cls.isMixin() || cls.isEnum()) return;

    boolean hasCtor = false;

    MethodDef[] methods = cls.methods();
    for (int i=0; i<methods.length; ++i)
      hasCtor |= methods[i].isCtor();

    if (!hasCtor)
      cls.slots.add(makeDefaultCtor(cls));
  }

  MethodDef makeDefaultCtor(ClassDef cls)
  {
    // ensure there isn't already a slot called make
    if (cls.hasSlot("make"))
      throw err("Default constructor 'make' conflicts with slot", cls.slot("make").location);

    return makeDefaultCtor(cls.location, FConst.Public);
  }

  static MethodDef makeDefaultCtor(ILocation loc, int flags)
  {
    Block block = new Block(loc);
//    block.statements.add(new SuperInitStmt(loc));
    block.statements.add(new ReturnStmt(loc));

    MethodDef m = new MethodDef(loc);
    m.flags = flags | FConst.Ctor;
    m.name = "make";
    m.ret = new TypeRef(loc, Sys.VoidType);
    m.code = block;
    return m;
  }

}