//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   2 Dec 05  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.io.*;
import java.util.*;
import fanx.util.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;

/**
 * Search the module's source tree for all the fan source files and
 * map them to SourceFile instances and store to Compiler.srcFiles.
 * Also find all the resource files and map them to Compiler.resFiles.
 * For right now, resource files are anything found in {srcDir}/res.
 * Summary:
 *   Compiler.srcDir\*.fan -> Compiler.srcFiles
 *   Compiler.srcDir\res\* -> Compiler.resFiles
 */
public class FindSourceFiles
  extends CompilerStep
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  public FindSourceFiles(Compiler compiler)
  {
    super(compiler);
  }

//////////////////////////////////////////////////////////////////////////
// Run
//////////////////////////////////////////////////////////////////////////

  public void run()
  {
    findSrcFiles();
    findResFiles();
  }

//////////////////////////////////////////////////////////////////////////
// srcFiles
//////////////////////////////////////////////////////////////////////////

  private void findSrcFiles()
  {
    ArrayList acc = new ArrayList();
    find(acc, compiler.srcDir, ".fan");
    File[] srcFiles = (File[])acc.toArray(new File[acc.size()]);
    compiler.srcFiles = srcFiles;

    if (srcFiles.length == 0)
      throw err("No fan source files found", compiler.srcDir);

    if (log().verbose)
    {
      log().verbose("SourceFiles", srcFiles.length + " files");
      log().indent();
      for (int i=0; i<srcFiles.length; ++i)
        log().verbose("", srcFiles[i]);
      log().unindent();
    }
  }

//////////////////////////////////////////////////////////////////////////
// srcFiles
//////////////////////////////////////////////////////////////////////////

  private void findResFiles()
  {
    ArrayList acc = new ArrayList();
    find(acc, new File(compiler.srcDir, "res"), "");
    File[] resFiles = (File[])acc.toArray(new File[acc.size()]);
    compiler.resFiles = resFiles;

    if (log().verbose)
    {
      log().verbose("ResourceFiles", resFiles.length + " files");
      log().indent();
      for (int i=0; i<resFiles.length; ++i)
        log().verbose("", resFiles[i]);
      log().unindent();
    }
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  private void find(ArrayList acc, File file, String ext)
  {
    if (!file.exists()) return;
    if (file.isDirectory())
    {
      File[] files = FileUtil.list(file);
      for (int i=0; files != null && i<files.length; ++i)
        find(acc, files[i], ext);
    }
    else
    {
      // only process .fan source files
      if (file.getName().endsWith(ext) && !file.getName().equals("build.fan"))
        acc.add(file);
    }
  }

}