//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   26 Dec 05  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.io.*;
import java.util.*;
import fan.sys.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;

/**
 * InitNameSpace is used to initialize the NameSpace with the compilation
 * unit ShimTypes.  We also do duplicate type name checking.
 */
public class InitNameSpace
  extends CompilerStep
{

  public InitNameSpace(Compiler compiler)
  {
    super(compiler);
  }

  public void run()
  {
    // init the NameSpace
    compiler.ns = new NameSpace(compiler);

    // hashes for duplication checks
    HashMap byName  = new HashMap();
    HashMap byLower = new HashMap();

    // process each ClassDef
    for (int i=0; i<compiler.classDefs.length; ++i)
    {
      ClassDef def = compiler.classDefs[i];
      ShimType type = def.type;
      String name = type.name().val;
      String lower = name.toLowerCase();

      // check for duplicates
      ClassDef dup = (ClassDef)byName.get(name);
      if (dup != null)
        throw err("Type \"" + name + "\" already defined in " + dup.location.format(), def);
      dup = (ClassDef)byLower.get(lower);
      if (dup != null)
        throw err("Type \"" + name + "\" differs only by case from " + dup.location.format(), def);

      // store for successive dup checks
      byName.put(name, def);
      byLower.put(lower, def);

      // add to NameSpace
      compiler.ns.add(type);
    }
  }

}