//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   2 Dec 05  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.io.*;
import java.util.*;
import fan.sys.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;

/**
 * InitShimExtends occurs after we have mapped each ClassDef to a ShimType,
 * and after we have resolved all the TypeRefs to their Types.  Now we
 * need to go thru all the extends clauses, and map them to our ShimType
 * data structures.
 */
public class InitShimExtends
  extends CompilerStep
{

  public InitShimExtends(Compiler compiler)
  {
    super(compiler);
  }

  public void run()
  {
    for (int i=0; i<compiler.classDefs.length; ++i)
    {
      ClassDef def = compiler.classDefs[i];
      ShimType type = (ShimType)def.type;
      if (def.base == null)
      {
        type.base = null;
      }
      else
      {
        type.base = def.base.type;
        if (def.base.type.isMixin().val) err("Cannot extend a mixin", def.base);
      }

      type.mixins = new Type[def.mixins.size()];
      for (int j=0; j<def.mixins.size(); ++j)
      {
        TypeRef ref = (TypeRef)def.mixins.get(j);
        type.mixins[j] = ref.type;
        if (!ref.type.isMixin().val) err("Cannot mixin a class", ref);
      }
    }
  }

}