//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Sep 05  Brian Frank  Creation
//
package fanx.test;

import fan.sys.*;

/**
 * CheckTypesTest
 */
public class CheckTypesTest
  extends CompileTest
{

//////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////

  public void doRun()
    throws Exception
  {
    // returns
    verifyErr("Str f() { return 4ns }",  "Cannot return 'sys::Duration' as 'sys::Str'");
    verifyErr("Void f() { return 4ns }", "Expected end of statement: semicolon, newline, or end of block; not '4ns'");
    verifyErr("Str f() { return }",      "Must return a value from non-void method");
    verifyErr("Str f() { return }",      "Must return a value from non-void method");

    // assignment
    verifyErr("Void f() { Int x; x = \"x\"; }", "Type 'sys::Str' is not assignable to 'sys::Int'");

    // local var init assignment
    verifyErr("Void f() { Str x := 5.0; }", "Type 'sys::Float' is not assignable to 'sys::Str'");

    // field init assignment
    verifyErr("Str x := 3ns; Void f() { }", "Type 'sys::Duration' is not assignable to 'sys::Str'");

    // bool !, &&, and || operators
    verifyErr("Void f() { !5 }", "Cannot apply '!' operator to 'sys::Int'");
    verifyErr("Void f() { 5 && true }", "Cannot apply '&&' operator to 'sys::Int' and 'sys::Bool'");
    verifyErr("Void f() { false && 6sec }", "Cannot apply '&&' operator to 'sys::Bool' and 'sys::Duration'");
    verifyErr("Void f() { 5 || true }", "Cannot apply '||' operator to 'sys::Int' and 'sys::Bool'");
    verifyErr("Void f() { false || 6sec }", "Cannot apply '||' operator to 'sys::Bool' and 'sys::Duration'");

    // is, as
    verifyErr("Bool f() { return true is this}",      "Expected type identifier, not 'this'");
    verifyErr("Obj f() { return true as Int.type}",   "Expected end of statement: semicolon, newline, or end of block; not '.'");
    verifyErr("Bool f() { return true is this.type}", "Expected type identifier, not 'this'");
  }

}