//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   22 Jan 06  Brian Frank  Creation
//
package fanx.test;

import fan.sys.*;

/**
 * ReflectTest
 */
public class ReflectTest
  extends CompileTest
{

//////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////

  public void doRun()
    throws Exception
  {
    verifyTypeParser();
    verifyBasic();
  }

//////////////////////////////////////////////////////////////////////////
// TypeParser
//////////////////////////////////////////////////////////////////////////

  public void verifyTypeParser()
    throws Exception
  {
    verify(Type.find("sys::Bool", true)     == Sys.BoolType);
    verify(Type.find("sys::Duration", true) == Sys.DurationType);
    verify(Type.find("sys::Int[]", true)    == Sys.IntType.toListOf());
    verifyTypeParserErr("");
    verifyTypeParserErr("x");
    verifyTypeParserErr("xy");
    verifyTypeParserErr("xyz");
    verifyTypeParserErr("x:z");
    verifyTypeParserErr("xz[");
    verifyTypeParserErr("xz[]");
    verifyTypeParserErr("[]");
  }

  public void verifyTypeParserErr(String sig)
  {
    try
    {
      Type.find(sig, false);
      System.out.println("didn't fail: " + sig);
      fail();
    }
    catch (Err.Val err)
    {
      //System.out.println(" -- " + err);
      verify(err.err().message().val.startsWith("Invalid type signature '" + sig + "'"));
    }
  }

//////////////////////////////////////////////////////////////////////////
// Basic
//////////////////////////////////////////////////////////////////////////

  public void verifyBasic()
    throws Exception
  {
    Type t = compileToType("class Foo { Str f() { return type.name } }");
    Str name = (Str)t.method("f", true).call1(t.make());
    verify(name.val.equals("Foo"));
  }

}