//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

using System;
using System.Reflection;

namespace Fan.Sys
{
  /// <summary>
  /// Field is a slot which "stores" a value.
  /// </summary>
  public class Field : Slot
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    public Field(Type parent, Str name, int flags, Facets facets, Type fieldType)
      : base(parent, name, flags, facets)
    {
      this.m_fieldType = fieldType;
    }

  //////////////////////////////////////////////////////////////////////////
  // Signature
  //////////////////////////////////////////////////////////////////////////

    public override Type type()  { return Sys.FieldType;  }

    public Type fieldType() { return m_fieldType; }

    public Method getter() { return m_getter; }

    public Method setter() { return m_setter; }

    public override Str signature() { return Str.make(m_fieldType.toStr().val + " " + Str.make(m_name.val)); }

  //////////////////////////////////////////////////////////////////////////
  // Reflection
  //////////////////////////////////////////////////////////////////////////

    public Obj get() { return get(null); }
    public Obj get(Obj instance)
    {
      if (m_getter != null)
      {
        return m_getter.invoke(instance, (Object[])null);
      }

      try
      {
        m_parent.finish();
        return (Obj)m_reflect.GetValue(instance);
      }
      catch (Exception e)
      {
        if (m_reflect == null)
          throw Err.make("Field not mapped to java.lang.reflect correctly").val;

        throw Err.make(e).val;
      }
    }

    public void set(Obj instance, Obj value)
    {
      if (m_setter != null)
      {
        m_setter.invoke(instance, new Obj[] { value });
        return;
      }

      try
      {
        m_parent.finish();
        m_reflect.SetValue(instance, value);
      }
      catch (Exception e)
      {
        if (m_reflect == null)
          throw Err.make("Field not mapped to java.lang.reflect correctly").val;

        throw Err.make(e).val;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    internal readonly Type m_fieldType;
    internal Method m_getter;
    internal Method m_setter;
    internal FieldInfo m_reflect;

  }
}