//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Jan 07  Andy Frank  Creation
//

namespace Fan.Sys
{
  /// <summary>
  /// File represents a file or directory in a file system.
  /// <summary>
  public abstract class File : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Construction
  //////////////////////////////////////////////////////////////////////////

    public static File make(Uri uri)
    {
      return new LocalFile(uri);
    }

    public static File os(Str osPath)
    {
      if (System.IO.Directory.Exists(osPath.val))
        return new LocalFile(new System.IO.DirectoryInfo(osPath.val));

      return new LocalFile(new System.IO.FileInfo(osPath.val));
    }

    public static File createTemp() { return createTemp(null, null, null); }
    public static File createTemp(Str prefix) { return createTemp(prefix, null, null); }
    public static File createTemp(Str prefix, Str suffix) { return createTemp(prefix, suffix, null); }
    public static File createTemp(Str prefix, Str suffix, File dir)
    {
      if (prefix == null || prefix.val.Length == 0) prefix = Str.make("fan");
      if (prefix.val.Length == 1) prefix = Str.make(prefix.val + "xx");
      if (prefix.val.Length == 2) prefix = Str.make(prefix.val + "x");

      if (suffix == null) suffix = Str.make(".tmp");

      //System.IO.FileSystemInfo d = null;
      //if (dir != null)
      //{
      //  if (!(dir is LocalFile)) throw IOErr.make("Dir is not on local file system: " + dir).val;
      //  d = ((LocalFile)dir).m_file;
      //}

      try
      {
        return null; //new LocalFile(java.io.File.createTempFile(prefix.val, suffix.val, d));
      }
      catch (System.IO.IOException e)
      {
        throw IOErr.make(e).val;
      }
    }

    protected File(Uri uri)
    {
      this.m_uri = uri;
    }

  //////////////////////////////////////////////////////////////////////////
  // .NET
  //////////////////////////////////////////////////////////////////////////

    public override int GetHashCode() { return m_uri.GetHashCode(); }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Bool equals(Obj obj)
    {
      if (obj is File)
      {
        return m_uri.equals(((File)obj).m_uri);
      }
      return Bool.False;
    }

    public override Int hash() { return m_uri.hash(); }

    public override Str toStr() { return m_uri.toStr(); }

    public override Type type() { return Sys.FileType; }

  //////////////////////////////////////////////////////////////////////////
  // Naming
  //////////////////////////////////////////////////////////////////////////

    public Uri uri() { return m_uri; }

    public Bool isDir() { return m_uri.isDir(); }

    public List path() { return m_uri.path(); }

    public Str pathStr() { return m_uri.pathStr(); }

    public Str name() { return m_uri.name(); }

    public Str basename() { return m_uri.basename(); }

    public Str ext() { return m_uri.ext(); }

  //////////////////////////////////////////////////////////////////////////
  // Access
  //////////////////////////////////////////////////////////////////////////

    public abstract Bool exists();

    public abstract Int size();

    public abstract Time modified();
    public abstract void modified(Time time);

    public abstract Str osPath();

    public abstract File parent();

    public abstract List list();

    public abstract File normalize();

    public abstract File plus(Uri uri);

  //////////////////////////////////////////////////////////////////////////
  // Management
  //////////////////////////////////////////////////////////////////////////

    public abstract File create();

    public abstract void delete();

    public abstract File deleteOnExit();

  //////////////////////////////////////////////////////////////////////////
  // IO
  //////////////////////////////////////////////////////////////////////////

    public InStream @in() { return @in(defaultBufSize); }
    public abstract InStream @in(Int bufSize);

    public OutStream @out() { return @out(Bool.False, defaultBufSize); }
    public OutStream @out(Bool append) { return @out(append, defaultBufSize); }
    public abstract OutStream @out(Bool append, Int bufSize);

    private static readonly Int defaultBufSize = Int.make(4096);

    public Buf readAllBuf()
    {
      return @in(defaultBufSize).readAllBuf();
    }

    public List readAllLines()
    {
      return @in(defaultBufSize).readAllLines();
    }

    public Str readAllStr() { return readAllStr(Bool.True); }
    public Str readAllStr(Bool normalizeNewlines)
    {
      return @in(defaultBufSize).readAllStr(normalizeNewlines);
    }

    public Map readProps()
    {
      return @in(defaultBufSize).readProps();
    }

    public void writeProps(Map props)
    {
      @out(Bool.False, defaultBufSize).writeProps(props, Bool.True);
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    internal readonly Uri m_uri;

  }
}