//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Sep 06  Andy Frank  Creation
//

using System;

namespace Fan.Sys
{
  /// <summary>
  /// MethodParam represents one formal parameter to a Method.
  /// </summary>
  public class MethodParam : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    public MethodParam(Str name, Type paramType, int mask)
    {
      this.m_name      = name;
      this.m_paramType = paramType;
      this.m_mask      = mask;
    }

  //////////////////////////////////////////////////////////////////////////
  // Methods
  //////////////////////////////////////////////////////////////////////////

    public override Type type() { return Sys.MethodParamType; }

    public Str name()        { return m_name; }
    public Type paramType()  { return m_paramType; }
    public Bool hasDefault() { return Bool.make(m_mask & HAS_DEFAULT); }

    public override Str toStr() { return Str.make(m_paramType + " " + m_name); }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    public static readonly int HAS_DEFAULT = 0x01;  // is a default value provided

    internal readonly Str m_name;
    internal readonly Type m_paramType;
    internal readonly int m_mask;

  }
}