//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jan 06  Andy Frank  Creation
//

using System.Collections;
using Fan.Sys;

namespace Fanx.Tools
{
  /// <summary>
  /// Fant .NET runtime.
  /// </summary>
  ///
  public class Fant : Tool
  {

  //////////////////////////////////////////////////////////////////////////
  // Test
  //////////////////////////////////////////////////////////////////////////

    public int test(string[] patterns, bool verbose)
    {
      long t1 = System.Environment.TickCount;
      for (int i=0; i<patterns.Length; i++)
        test(patterns[i], verbose);
      long t2 = System.Environment.TickCount;

      writeLine("");
      writeLine("Time: " + (t2-t1) + "ms");
      writeLine("");
      writeLine("***");
      writeLine("*** " + (failures == 0 ? "All tests passed!" : failures + " FAILURES") + " [" + testCount + " tests, " + methodCount  + " methods, " + totalVerifyCount + " verifies] ");
      writeLine("***");
      return failures;
    }

    public void test(string pattern, bool verbose)
    {
      // TODO
      /*
      if (pattern == "*")
      {
        List pods = Pod.List();
        for (int i=0; i<pods.Sz(); i++)
        {
          Pod pod = (Pod)pods.Get(i);
          Test(pod.Name().val, verbose);
        }
        return;
      }
      */

      // TODO
      /*
      if (pattern == "sys" || pattern.StartsWith("sys::"))
      {
        WriteLine("");
        if (pattern == "sys")
          pattern = "";
        else
          pattern = pattern.Substring(5);
        if (!Fanx.Test.Test.Test(pattern)) failures++;
        totalVerifyCount += Fanx.Test.Test.totalVerified;
        return;
      }
      */

      string[] s = pattern.Split(new char[] { ':', '.' });
      string podName = s[0];
      string testName = (s.Length > 2) ? s[2] : "*";
      string methodName = (s.Length > 3) ? s[3] : "*";

      writeLine("");

      Pod pod = Pod.find(podName, true, null);
      Type[] t = tests(pod, testName);
      for (int i=0; i<t.Length; i++)
      {
        Method[] m = methods(t[i], methodName);
        for (int j=0; j<m.Length; j++)
        {
          writeLine("-- Run:  " + m[j] + "...");
          int verifyCount = runTest(t[i], m[j]);
          if (verifyCount < 0) { failures++; continue; }
          writeLine("   Pass: " + m[j] + " [" + verifyCount + "]");
          methodCount++;
          totalVerifyCount += verifyCount;
        }
        testCount++;
      }
    }

    private Type[] tests(Pod pod, string testName)
    {
      // named test
      if (testName != "*") return new Type[] { pod.findType(testName, true) };

      // all types which subclass Test
      List all = pod.types();
      ArrayList acc = new ArrayList();
      for (int i=0; i<all.sz(); i++)
      {
        Type x = (Type)all.get(i);
        // TODO - is?
        //if (x.Is(Sys.TestType) && !x.IsAbstract().val) acc.Add(x);
        acc.Add(x);
      }
      return (Type[])acc.ToArray(System.Type.GetType("Fan.Sys.Type"));
    }

    private Method[] methods(Type type, string methodName)
    {
      // named test
      if (methodName != "*") return new Method[] { type.method(methodName, true) };

      // all methods which start with "test"
      Method[] all = type.methodArr();
      ArrayList acc = new ArrayList();
      for (int i=0; i<all.Length; i++)
        if (all[i].name().val.StartsWith("test") && !all[i].isAbstract().val) acc.Add(all[i]);
      return (Method[])acc.ToArray(System.Type.GetType("Fan.Sys.Method"));
    }

    private int runTest(Type type, Method method)
    {
      Method setup    = type.method("setup", true);
      Method teardown = type.method("teardown", true);

      try
      {
        Fan.Sys.Test test = (Fan.Sys.Test)type.make();
        List args = new List(Sys.ObjType, new Obj[] {test});
        setup.call(args);
        method.call(args);
        teardown.call(args);
        return test.verifyCount;
      }
      catch (System.Exception e)
      {
        System.Console.WriteLine();
        System.Console.WriteLine("TEST FAILED");
        if (e is Err.Val)
          ((Err.Val)e).err().trace();
        else
        {
          System.Console.WriteLine(e);
          System.Console.WriteLine(e.StackTrace);
        }
        return -1;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Tool Methods
  //////////////////////////////////////////////////////////////////////////

    public override int run() //string[] args)
    {
      try
      {
        //Fanx.Test.Test.RunTests(null);
        //Test(new string[] { "test::BoolTest.testEquals" }, true);
        //Test(new string[] { "test::UriTest" }, true);
        //Test(new string[] { "test" }, true);

        ///*
        test(new string[]
        {
          "test::BoolTest",
          "test::CallTest",
          "test::DurationTest",
          "test::FloatTest",
          "test::IntTest",
          "test::ListTest",
          "test::MapTest",
          "test::RangeTest",
          "test::StmtTest",
          "test::StrTest",
          "test::TimeTest",
          "test::EnumTest",
          "test::TypeTest",
          "test::MixinTest",
          "test::ExprTest",
          "test::ObjTest",
          "test::ReflectionTest",
          "test::VersionTest",
          "test::DependTest",
          "test::CtorTest",
          "test::ClosureTest",
          "test::FieldTest",
          "test::UriTest",     // lots to do here!
          "test::ParamTest",   // need to fix all this!
          "test::CharsetTest",
          "test::StreamTest",
          "test::BufTest",
          "test::FileTest",    // a few things here
          "test::ErrTest",     // testSubclassing
          "test::MethodTest",  // testReflectClosures
          "test::PodTest",
          "test::ZipTest",
          "test::SwitchTest",
          "test::SysTest",
          "test::TryTest",     // testRuntimeMapping
          "test::SlotTest",
          "test::StrBufTest",
          "test::FacetsTest",
        }, true);
        //*/

        //Type type = Type.Find("sysTest::BoolTest");
        //Type type = Type.Find("test::BoolTest");
        //Method[] m = Methods(type, "*");
        //for (int i=0; i<m.Length; i++)
        //  System.Console.WriteLine("[" + i + "] " + m[i]);
        //  RunTest(type, m[i]);
      }
      catch (System.Exception e)
      {
        System.Console.WriteLine(e);
        System.Console.WriteLine(e.StackTrace);
        return -1;
      }

      return 0;

      // TODO
      //Help();
      //return -1;
    }

    void help()
    {
      writeLine("Fan Test");
      writeLine("Usage:");
      writeLine("  fant [options] <pod> [pod]*");
      writeLine("  fant [options] <pod>::<test>");
      writeLine("  fant [options] <pod>::<test>.<method>");
      writeLine("Options:");
      writeLine("  -help, -h, -?  print usage help");
      writeLine("  -v             verbose mode");
    }

    void writeLine(string s)
    {
      System.Console.WriteLine(s);
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    int testCount = 0;
    int methodCount = 0;
    int totalVerifyCount = 0;
    int failures = 0;
  }
}
