//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Apr 06  Brian Frank  Creation
//

**
** Enum is the base class for enum classes defined using the 'enum'
** keyword.  An enum models a fixed range of discrete values.  Each
** value has an Int ordinal and a Str name.  Every enum class implicitly
** has a static field called "values" which contains the list of discrete
** values indexed by ordinal.
**
** TODO: examples (with ctor), name lookup
**
abstract const class Enum
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  **
  ** Protected constructor - for compiler use only.
  **
  protected new make(Int ordinal, Str name)

//////////////////////////////////////////////////////////////////////////
// Obj Overrides
//////////////////////////////////////////////////////////////////////////

  **
  ** Enums are only equal if same instance using ===.
  **
  override Bool equals(Obj obj)

  **
  ** Compare based on ordinal value.
  **
  override Int compare(Obj obj)

  **
  ** Default implementation returns name().
  **
  override virtual Str toStr()

/////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  **
  ** Get the programatic name for this enum.
  **
  Str name()

  **
  ** Return ordinal value which is a zero based index into values.
  **
  Int ordinal()

}