//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jul 06  Brian Frank  Creation
//

**
** Facets encapsulate meta-data as a map of Str name/value pairs.
**
final class Facets
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  **
  ** Make empty facets.
  **
  new make()

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  **
  ** Two facets are equal if they have equal maps.
  ** This method is idempotent.
  **
  override Bool equals(Obj that)

  **
  ** Return hash code based on map of name/value pairs.
  ** This method is idempotent.
  **
  override Int hash()

  **
  ** Return string representation.
  ** This method is idempotent.
  **
  override Str toStr()

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  **
  ** Return if size is zero.
  ** This method is idempotent.
  **
  Bool isEmpty()

  **
  ** Return number of key/value pairs.
  ** This method is idempotent.
  **
  Int size()

  **
  ** Get list of string keys.
  ** This method is idempotent.
  **
  Str[] keys()

//////////////////////////////////////////////////////////////////////////
// Getters
//////////////////////////////////////////////////////////////////////////

  **
  ** Get a facet by name or def if not mapped.
  ** This method is idempotent.
  **
  Str get(Str key, Str def := null)

  **
  ** Return get(name).toBool or def if not mapped.
  ** This method is idempotent.
  **
  Bool getBool(Str key, Bool def := null)

  **
  ** Return get(name).toInt or def if not mapped.
  ** This method is idempotent.
  **
  Int getInt(Str key, Int def := null)

  **
  ** Return get(name).toFloat or def if not mapped.
  ** This method is idempotent.
  **
  Float getFloat(Str key, Float def := null)

//////////////////////////////////////////////////////////////////////////
// Modifiers
//////////////////////////////////////////////////////////////////////////

  **
  ** Add or replace a facet with the specified name/value pair.
  ** Return this.  Throw ReadonlyErr if readonly.
  **
  Facets set(Str key, Str buf)

  **
  ** Convenience for set(name, val.toStr).
  ** Return this.  Throw ReadonlyErr if readonly.
  **
  Facets setBool(Str key, Bool val)

  **
  ** Convenience for set(name, val.toStr).
  ** Return this.  Throw ReadonlyErr if readonly.
  **
  Facets setInt(Str key, Int val)

  **
  ** Convenience for set(name, val.toStr).
  ** Return this.  Throw ReadonlyErr if readonly.
  **
  Facets setFloat(Str key, Float val)

  **
  ** Remove the specified key and return its old value.
  ** Throw ReadonlyErr if readonly.
  **
  Str remove(Str key)

  **
  ** Remove all key/value pairs from the map.  Throw
  ** ReadonlyErr if readonly.
  **
  Void clear()

//////////////////////////////////////////////////////////////////////////
// Readonly
//////////////////////////////////////////////////////////////////////////

  **
  ** Return if these Facets are readonly.  Readonly Facets are
  ** guaranteed to be immutable.  Any attempt to modify readonly Facets
  ** will result in ReadonlyErr.  Use rw() to get read-write Facets
  ** from readonly Facets.  Methods documented as idempotent may be
  ** used safely with readonly Facets.  This method is idempotent.
  **
  Bool isRO()

  **
  ** Return if these Facets are read-write.  Read-write Facets are
  ** mutable and may be modified.  Use ro() to get readonly Facets
  ** from read-write Facets.  This method is idempotent.
  **
  Bool isRW()

  **
  ** Get a readonly, immutable Facets instance with the same contents
  ** as these Facets.  If these Facets are already readonly, then return
  ** this.  Only methods documented as idempotent may be used safely with
  ** readonly Facets, all others will throw ReadonlyErr.  This method
  ** is idempotent.
  **
  Facets ro()

  **
  ** Get a read-write, mutable Facets instance with the same contents
  ** as these Facets.  If these Facets are already read-write, then
  ** return this.  This method is idempotent.
  **
  Facets rw()

}