//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   6 Jan 06  Brian Frank  Creation
//

**
** Field is a slot used to store an object.
**
final class Method : Slot
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  **
  ** Private constructor.
  **
  private new make()

//////////////////////////////////////////////////////////////////////////
// Signature
//////////////////////////////////////////////////////////////////////////

  **
  ** Type returned by the method or sys::Void if no return value.
  **
  Type returnType()

  **
  ** Get the parameter signature of the method.
  **
  MethodParam[] params()

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  **
  ** Dynamically invoke this method with the specified arguments and return
  ** the result.  If the method has Void return type, then null is returned.
  ** The argument list must match the number and type of required parameters.
  ** If this Method represents an instance method (not static and not a
  ** constructor) then the first argument must be the target object.  If the
  ** method supports default parameters, omit arguments to use the defaults.
  ** It is permissible to pass more arguments then the number of method
  ** parameters - the additional arguments are ignored.  If no arguments are
  ** required, you may pass null for args.
  **
  virtual R call(Obj[] args)

  **
  ** Convenience for dynamically invoking an instance method with
  ** specified target and arguments.  This method is semantically
  ** equivalent to call([target, args[0], args[1] ...]).  If callOn
  ** is used with a static method, then the target is ignored.  If
  ** no arguments are required, you may pass null for args.
  **
  virtual R callOn(Obj target, Obj[] args)

  **
  ** Optimized convenience for call([,]).
  **
  virtual R call0()

  **
  ** Optimized convenience for call([a]).
  **
  virtual R call1(A a)

  **
  ** Optimized convenience for call([a, b]).
  **
  virtual R call2(A a, B b)

  **
  ** Optimized convenience for call([a, b, c]).
  **
  virtual R call3(A a, B b, C c)

  **
  ** Optimized convenience for call([a, b, c, d]).
  **
  virtual R call4(A a, B b, C c, D d)

  **
  ** Optimized convenience for call([a, b, c, d, e]).
  **
  virtual R call5(A a, B b, C c, D d, E e)

  **
  ** Optimized convenience for call([a, b, c, d, e, f]).
  **
  virtual R call6(A a, B b, C c, D d, E e, F f)

  **
  ** Optimized convenience for call([a, b, c, d, e, f, g]).
  **
  virtual R call7(A a, B b, C c, D d, E e, F f, G g)

  **
  ** Optimized convenience for call([a, b, c, d, e, f, g, h]).
  **
  virtual R call8(A a, B b, C c, D d, E e, F f, G g, H h)

}