//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Jul 06  Brian Frank  Creation
//

**
** Time represents an absolute instance in time.  Fan time is normalized
** as nanosecond ticks since 1 Jan 1970  GMT.
**
const final class Time
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  **
  ** Return the current time.  The tolerance parameter specifies
  ** that you are willing to use a cached Time instance as long as
  ** (now - cached <= tolerance).  If tolerance is null, then this
  ** method always creates a new Time instance.  Using tolerance
  ** can increase performance and save memory.  The tolerance
  ** default is 250ms.
  **
  ** If you are using time to calculate relative time periods,
  ** then use Duration.now instead.  Duration is more efficient
  ** and won't cause you grief when the system clock is modified.
  **
  static Time now(Duration tolerance := 250ms)

  **
  ** Make for nanosecond ticks since 1 Jan 1970 GMT.
  **
  new make(Int ticks)

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  **
  ** Two times are equal if have identical nanosecond ticks.
  **
  override Bool equals(Obj that)

  **
  ** Return nanosecond ticks for the hashcode.
  **
  override Int hash()

  **
  ** Compare based on nanosecond ticks.
  **
  override Int compare(Obj obj)

  **
  ** Return string representation.
  **
  override Str toStr()

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  **
  ** Return number of nanosecond ticks since 1 Jan 1970.
  **
  Int ticks()

  // TODO: just about everything

//////////////////////////////////////////////////////////////////////////
// Operators
//////////////////////////////////////////////////////////////////////////

  **
  ** Return the delta between this and the given time.
  **
  ** Example:
  **   elapsed := Time.now - startTime
  **
  Duration minus(Time time)

  **
  ** Add (or subtract if duration is negative) a duration
  ** to compute a new time.
  **
  ** Example:
  **   tomorrow  := Time.now + 1days
  **   yesterday := Time.now + -1days
  **
  Time plus(Duration duration)

//////////////////////////////////////////////////////////////////////////
// HTTP
//////////////////////////////////////////////////////////////////////////

  **
  ** Parse an HTTP date according to the RFC 2616 section 3.3.1.  If
  ** invalid format and checked is false return null, otherwise
  ** throw ArgErr.  The following date formats are supported:
  **
  **   Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
  **   Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
  **   Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
  **
  static Time parseHttp(Str s, Bool checked := true)

  **
  ** Format this time for use in an MIME or HTTP message according
  ** to RFC 2616 section 3.3.1:
  **
  **  Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
  **
  Str formatHttp()


}